/// <reference types="bun-types" />
/// <reference types="node" />

import { AsyncReturnType } from 'type-fest';
import { DefaultArgs } from '@prisma/client/runtime/library';
import { Duplex } from 'node:stream';
import { IncomingMessage } from 'node:http';
import { Prisma } from '.prisma/client';
import { PrismaClient } from '@prisma/client';
import type { ServerWebSocket } from 'bun';
import { SetReturnType } from 'type-fest';
import type { default as WebSocket_2 } from 'ws';

declare const all: <T extends [] | readonly Promise<unknown>[]>(promises: T) => BatchedPromise<T, "all">;
export { all }
export { all as all_alias_1 }

declare const allSequential: <T extends [] | readonly Promise<unknown>[]>(promises: T) => BatchedPromise<T, "allSequential">;
export { allSequential }
export { allSequential as allSequential_alias_1 }

declare const allSettled: <T extends [] | readonly Promise<unknown>[]>(promises: T) => BatchedPromise<T, "allSettled">;
export { allSettled }
export { allSettled as allSettled_alias_1 }

declare const allSettledSequential: <T extends [] | readonly Promise<unknown>[]>(promises: T) => BatchedPromise<T, "allSettledSequential">;
export { allSettledSequential }
export { allSettledSequential as allSettledSequential_alias_1 }

declare type AnyClientMethod = {
    (...args: any[]): Promise<any>;
    [Symbol.toStringTag]: string;
};
export { AnyClientMethod }
export { AnyClientMethod as AnyClientMethod_alias_1 }
export { AnyClientMethod as AnyClientMethod_alias_2 }

declare const _args: unique symbol;

export declare const assertRequestData: (data: unknown) => RequestData;

declare type AsyncData<Fn extends Handler> = ReturnType<Fn> extends void ? SetReturnType<Fn, Promise<void>> : ReturnType<Fn> extends Subscribe<infer U> ? (...args: [...Parameters<Fn>, handler: SubjectHandler<U>]) => () => void : ReturnType<Fn> extends WithResponseInit<infer U> ? AsyncData<SetReturnType<Fn, U>> : ReturnType<Fn> extends Promise<unknown> ? Fn : SetReturnType<Fn, Promise<AsyncReturnType<Fn>>>;

declare type BatchedPromise<T extends readonly Promise<unknown>[] | [], Type extends Exclude<RequestData['type'], 'request'>> = IntegroPromise<{
    -readonly [P in keyof T]: Type extends 'all' | 'allSequential' ? Awaited<T[P]> : PromiseSettledResult<Awaited<T[P]>>;
}, [BatchPostOptions]> & [...{
    -readonly [P in keyof T]: IntegroPromise<Awaited<T[P]>>;
}, () => BatchedPromise<T, Type>];
export { BatchedPromise }
export { BatchedPromise as BatchedPromise_alias_1 }

export declare type BatchPostOptions = {
    url: string;
    config: ClientConfig;
    data: BatchRequestData;
};

export declare type BatchRequestData = {
    type: 'all' | 'allSettled' | 'allSequential' | 'allSettledSequential';
    data: RequestData[];
};

export declare class BodyParsingError extends Error {
}

declare type ClientConfig = {
    auth?: string | (() => string | undefined);
    requestInit?: RequestInit | (() => RequestInit);
    subscribeKey?: string;
};
export { ClientConfig }
export { ClientConfig as ClientConfig_alias_1 }

declare const createClient: <T extends IntegroApp>(url?: string, config?: ClientConfig) => IntegroClient<T>;
export { createClient }
export { createClient as createClient_alias_1 }

declare const createController: (app: IntegroApp) => RequestHandler;
export { createController }
export { createController as createController_alias_1 }

export declare const createIntegroPromise: <T, Args extends unknown[] = unknown[]>(executor: LazyExecuter<T, Args>, ...args: Args) => IntegroPromise<T, Args>;

export declare const createProxy: <T extends object | Handler, U extends (path: string[], args: unknown[]) => unknown = (path: string[], args: unknown[]) => unknown>(apply: U, path?: string[]) => T;

declare const createSubject: <T>(config?: SubjectConfig) => Subject<T>;
export { createSubject }
export { createSubject as createSubject_alias_1 }

declare const createSubscriptionController: (app: IntegroApp, { subscribeKey }?: SubscriptionControllerConfig) => {
    createWebSocketServer: (request: IncomingMessage, socket: Duplex, head: Buffer) => void;
    handleRequest: (request: Request | IncomingMessage, arg2?: unknown) => Promise<Response>;
    unsubscribeAll: () => void;
    websocketHandlers: {
        close: () => void;
        message: (ws: ServerWebSocket | WebSocket_2, message: WebSocket_2.RawData | string) => Promise<void>;
        open: (ws: ServerWebSocket | WebSocket_2) => void;
    };
};
export { createSubscriptionController }
export { createSubscriptionController as createSubscriptionController_alias_1 }

declare const defineApp: <T extends IntegroApp>(app: T) => T;
export { defineApp }
export { defineApp as defineApp_alias_1 }

export declare const everyItemIsString: (array: unknown[]) => array is string[];

export declare const getIntegroPromiseArgs: <T extends IntegroPromise<unknown, unknown[]>>(promise: T) => unknown[];

declare type Handler = (...arguments_: any[]) => any;
export { Handler }
export { Handler as Handler_alias_1 }
export { Handler as Handler_alias_2 }

export declare type HandlerContext = MessageContext | RequestContext;

declare type IntegroApp = Handler | Promise<Handler> | {
    [K: string]: IntegroApp | Promise<IntegroApp>;
};
export { IntegroApp }
export { IntegroApp as IntegroApp_alias_1 }
export { IntegroApp as IntegroApp_alias_2 }

declare type IntegroClient<T extends IntegroApp> = T extends Unwrappable<infer U> ? IntegroClient<U> : T extends Subject<infer U> ? {
    subscribe: (handler: SubjectHandler<U>) => () => void;
} : T extends Handler ? (AsyncData<T> & {
    [Symbol.toStringTag]: string;
}) : {
    [K in keyof T]: T[K] extends IntegroApp ? IntegroClient<T[K]> : never;
};
export { IntegroClient }
export { IntegroClient as IntegroClient_alias_1 }
export { IntegroClient as IntegroClient_alias_2 }

export declare type IntegroPromise<T, Args extends unknown[] = unknown[]> = Promise<T> & {
    [_args]: Args;
    [_setPromise]?: (promise: Promise<T>) => void;
};

export declare const isArrayEqual: (a: unknown[], b: unknown[]) => boolean;

export declare const isIntegroPromise: (promise: unknown) => promise is IntegroPromise<unknown, unknown[]>;

export declare const isIntegroPromises: (promises: readonly unknown[]) => promises is IntegroPromise<unknown, unknown[]>[];

export declare const isObject: (object: unknown) => object is object;

export declare const isResponseInitObject: <T>(object: WithResponseInit<T> | unknown) => object is WithResponseInit<T>;

declare const _isResponseInitObject: unique symbol;

export declare const isSubject: <T>(object: unknown) => object is Subject<T>;

export declare const isUnwrappable: <T extends IntegroApp>(fn: unknown) => fn is Unwrappable<T>;

declare const _isUnwrappable: unique symbol;

declare type LazyExecuter<T, Args extends unknown[]> = (...args: Args) => Promise<T> | T;

export declare const merge: <T extends object>([first, second, ...rest]: T[]) => T;

export declare type MessageContext = {
    type: 'message';
    auth?: string;
    request?: undefined;
};

export declare const nestedApp: {
    getName: () => string;
    nestedFunction: Unwrappable<() => string>;
};

export declare const nestedFunction: () => string;

export declare const noop: () => void;

export declare class PathError extends Error {
}

export declare const pipe: <T = unknown>(value: T, next?: (value: T) => T, ...rest: ((value: T) => T)[]) => T;

export declare const post: (options: PostOptions) => Promise<any>;

export declare type PostOptions = {
    url: string;
    config: ClientConfig;
    data: RequestData;
};

export declare const prisma: PrismaClient<Prisma.PrismaClientOptions, never, DefaultArgs>;

export declare const rawDataToBuffer: (data: ArrayBuffer | Buffer | Buffer[]) => Buffer;

export declare type RegularRequestData = {
    type: 'request';
    path: string[];
    args: unknown[];
};

export declare type RequestContext = {
    type: 'request';
    auth?: undefined;
    request: Request;
};

export declare type RequestData = RegularRequestData | BatchRequestData;

declare type RequestHandler = (request: IncomingMessage | Request, arg2?: unknown) => Promise<Response>;

export declare const resolveProp: ({ path: [firstProp, ...path], app, context }: {
    path: string[];
    app: IntegroApp;
    context: HandlerContext;
}) => Promise<IntegroApp>;

declare const respondWith: <T>(data?: T, responseInit?: ResponseInit) => WithResponseInit<T>;
export { respondWith }
export { respondWith as respondWith_alias_1 }

export declare const setIntegroPromiseValue: <T>(promise: IntegroPromise<T, unknown[]>, nativePromise: Promise<T>) => void;

declare const _setPromise: unique symbol;

declare type Subject<T> = {
    _subscriptionCount: () => number;
    filter: (filter: (message: T) => boolean) => Subject<T>;
    map: <U = T>(mapper: (message: T) => U) => Subject<U>;
    send: (message: T) => void;
    subscribe: Subscribe<T>;
};
export { Subject }
export { Subject as Subject_alias_1 }

declare type SubjectConfig = {
    onUnsubscribe?: (remainingSubscriptionCount: number) => void;
};
export { SubjectConfig }
export { SubjectConfig as SubjectConfig_alias_1 }

declare type SubjectHandler<T> = (message: T, error?: {
    message: string;
}) => void;
export { SubjectHandler }
export { SubjectHandler as SubjectHandler_alias_1 }

declare type Subscribe<T> = (handler: SubjectHandler<T>) => () => void;
export { Subscribe }
export { Subscribe as Subscribe_alias_1 }

declare type SubscriptionControllerConfig = {
    subscribeKey?: string;
};
export { SubscriptionControllerConfig }
export { SubscriptionControllerConfig as SubscriptionControllerConfig_alias_1 }

export declare const toRequest: (incomingMessage: IncomingMessage) => Promise<Request>;

declare const unwrap: <T extends IntegroApp>(handler: WrappedHandler<T>) => Unwrappable<T>;
export { unwrap }
export { unwrap as unwrap_alias_1 }

declare type Unwrappable<T extends IntegroApp> = WrappedHandler<T> & {
    [_isUnwrappable]: true;
};
export { Unwrappable }
export { Unwrappable as Unwrappable_alias_1 }

declare type WithResponseInit<T> = {
    [_isResponseInitObject]: true;
    data?: T;
    responseInit?: ResponseInit;
};
export { WithResponseInit }
export { WithResponseInit as WithResponseInit_alias_1 }

declare type WrappedHandler<T extends IntegroApp> = (context: HandlerContext) => T | Promise<T>;
export { WrappedHandler }
export { WrappedHandler as WrappedHandler_alias_1 }

export { }
